"""
Yandex Image Crawler - A professional image crawling tool using Yandex's image search.
"""

from .crawler import YandexImageCrawler
from .version import __version__

__all__ = ['YandexImageCrawler', '__version__'] 